# -*- Mode: Perl -*-

#
# Copyright (c) 2001, Bryan Jurish.  All rights reserved.
#
# This package is free software.  You may redistribute it
# and/or modify it under the same terms as Perl itself.
#

###############################################################
#
#      File: Data::FStructure::Std.pm
#    Author: Bryan Jurish <jurish@ling.uni-potsdam.de>
#
# Description: Feature-structures for perl
#
###############################################################

package Data::FStructure::Std;
# System modules
# 3rd party exstensions
# user extension modules
use Data::FStructure::Base;
use Data::FStructure::theta;
use Data::FStructure::delta;
@ISA       = qw(Data::FStructure::Base);
#@EXPORT   = qw();
#@EXPORT_OK = (
#	     );
#%EXPORT_TAGS =
#  (
#  );

###############################################################
# Package-global variables
###############################################################
our $VERSION = 0.01;

#our $INCORPORATE = 1;
#our $DEBUG = 1;

###############################################################
# Constructor
#   + Data::FStructure::Std->new( {hierarchy=>$h} )
#   + Object-structure:
#     { --- inherited ---
#       hierarchy       => Math::PartialOrder
#       Q (nodes)       => { id1 => node1, ... } (blessed as Set::Hashed)
#       q0 (root)       => Scalar node
#       qcounter        => Scalar integer (counts nodes)
#       pathsep (opt)   => Scalar path-separator
#       pathre (opt)    => Scalar pathsep-regexp
#       master          => Scalar-ref: root of largest embedding FStructure
#      --- new ---
#       theta (types)   => qd::fs::theta (new!)
#       delta (ftrs)    => qd::fs::delta (new!)
#     }
#   + Args:
#     { hierarchy => $h,
#       qcounter => $i,
#       pathsep => $str,
#       pathsplit => $regex,
#       root => $nodeid
#       theta => Data::FStructure::theta,
#       delta => Data::FStructure::delta,
#     }
###############################################################
sub initialize {
  my $self = shift;
  $self->{theta} = Data::FStructure::theta->new()
    unless (defined($self->{theta}));
  $self->{delta} = Data::FStructure::delta->new()
    unless (defined($self->{delta}));
  return $self;
}

###############################################################
# Accessors/manipulators
###############################################################

# $th = $fs->_theta;
# $fs->_theta($th);
sub _theta {
  return exists($_[1]) ? $_[0]->{theta} = $_[1] : $_[0]->{theta};
}

# $dt = $fs->_delta;
# $fs->_delta($dt);
sub _delta {
  return exists($_[1]) ? $_[0]->{delta} = $_[1] : $_[0]->{delta};
}



1;
__END__

# -*- Mode: Perl -*-
=pod

=head1 NAME

Data::FStructure::Std - Standard feature structures for perl.

=head1 SYNOPSIS

  use Data::FStructure;

  #
  # Construction and Initialization
  #
  $fs = Data::FStructure::Std->new(\%args);  # return a new feature-structure

  $th = $fs->_theta();                      # get type map
  $fs->_theta($th);                         # set type map

  $dt = $fs->_delta();                      # get transitition map
  $fs->_delta($dt);                         # set transition map

=head1 REQUIREMENTS

Carp,

=cut

#--------------------------------------------------------------
# Description
#--------------------------------------------------------------
=pod

=head1 DESCRIPTION


=cut

#--------------------------------------------------------------
# Variables
#--------------------------------------------------------------
#=pod
#
#=head1 PACKAGE VARIABLES
#
#=cut


#--------------------------------------------------------------
# Methods
#--------------------------------------------------------------
=pod

=head1 METHODS

=over 4

=item * C<new()>

Just like the C<Data::FStructure::Base> method C<new()>, but
C<theta> and C<delta> default to
new C<Data::FStructure::theta> and C<Data::FStructure::delta> objects.

=item * C<_theta()>, C<_theta($th)>

Get/set the C<Data::FStructure::theta> object representing the
type-mapping for the FStructure object.

=item * C<_delta()>, C<_delta($dt)>

Get/set the C<Data::FStructure::delta> object representing the
transition-mapping for the FStructure object.

=back

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2001, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).
Data::FStructure(3pm).
Data::FStructure::Base(3pm).
Data::FStructure::theta(3pm).
Data::FStructure::delta(3pm).
