# -*- Mode: Perl -*-

#
# Copyright (c) 2001, Bryan Jurish.  All rights reserved.
#
# This package is free software.  You may redistribute it
# and/or modify it under the same terms as Perl itself.
#

###############################################################
#
#      File: Data::FStructure::perldata.pm
#    Author: Bryan Jurish <jurish@ling.uni-potsdam.de>
#
# Description: Feature-structures for perl :
#              convert to/from standard perl data-structures
#
###############################################################

package Data::FStructure::perldata;
use Data::FStructure::Base;
use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = (
	      qw($GOBBLE_SHARE_SCALARS $GOBBLE_CALL_CODEREFS $GOBBLE_CALL_METHODS),
	      qw($GOBBLE_UNTIE_HASHES $GOBBLE_UNTIE_ARRAYS $GOBBLE_REFVAL_FTR),
	      qw($GOBBLE_ON_PVALUE $GOBBLE_USE_APPROP $GOBBLE_REUSE_PATHS),
	      qw($GOBBLE_WANT_LINKED_LISTS),
	      qw(%SPIT_TYPE2REF %SPIT_LINKUS),
	     );
%EXPORT_TAGS =
  (
   'gobblevars' => [qw($GOBBLE_SHARE_SCALARS $GOBBLE_CALL_CODEREFS $GOBBLE_CALL_METHODS),
		    qw($GOBBLE_UNTIE_HASHES $GOBBLE_UNTIE_ARRAYS $GOBBLE_REFVAL_FTR),
		    qw($GOBBLE_ON_PVALUE $GOBBLE_USE_APPROP $GOBBLE_REUSE_PATHS),
		    qw($GOBBLE_WANT_LINKED_LISTS),
		   ],
   'spitvars' => [qw(%SPIT_TYPE2REF %SPIT_LINKUS)]
  );

our $VERSION = 0.03;

our $GOBBLE_SHARE_SCALARS = 1;
our $GOBBLE_CALL_CODEREFS = 1;
our $GOBBLE_CALL_METHODS = 1;
our $GOBBLE_UNTIE_HASHES = 1;
our $GOBBLE_UNTIE_ARRAYS = 1;
our $GOBBLE_USE_APPROP = 1;
our $GOBBLE_REUSE_PATHS = 1;
our $GOBBLE_REFVAL_FTR = 'VALUE';
our $GOBBLE_WANT_LINKED_LISTS = '';

our %SPIT_TYPE2REF =
  (
   DEFAULT => \&_type2ref_default,
  );
our %SPIT_LINKUS =
  (
   DEFAULT => \&_linkus_default,
  );


# &type2ref($fs,$type,$node,\%node2ref_in_progress)
sub _type2ref_default {
  my ($ary);
  if ($_[0]->{delta}->isa('Data::FStructure::arydelta')
      && defined($ary = $_[0]->nodeary($_[2]))
      && ref($ary) && ref($ary) eq 'ARRAY') {

    # it *might* be a valid array -- check
    foreach ($_[0]->{delta}->labels_from($_[2])) {
      last if ($_ !~ /^\d+$/);  # ... oops
    }
    # ... whew
    return
      (defined($_[1]) && $_[1] ne 'ARRAY'
       ? bless([], $_[1])
       : []);
  }
  return
    (defined($_[1]) && 
     $_[1] ne 'ARRAY' && $_[1] ne 'HASH' && $_[1] ne 'REF' && $_[1] ne 'CODE'
     ? bless({},$_[1])
     : {});
}

# undef = &linkus_callback($fs,$qfrom,$f,$qto,\%node2ref);
sub _linkus_default {
  my ($fs,$from,$f,$to,$node2ref) = @_;
  if (UNIVERSAL::isa($node2ref->{$from},'ARRAY')) {
    # we can now assume it's good
    $node2ref->{$from}->[$f] = $node2ref->{$to};
    return;
  }
  # do the (old) default thing...
  $_[4]->{$_[1]}{$_[2]} = $_[4]->{$_[3]};
}


###############################################################
# sneakily switch packages
###############################################################
package Data::FStructure::Base;
Data::FStructure::perldata->import qw(:gobblevars :spitvars);

###############################################################
# spit
###############################################################

# $data = $fs->spit();
# $data = $fs->spit(%opts);
# + %opts:
#    from => $node=$fs->root;
#    type2ref => { type => \&sub, ..., DEFAULT => \&sub }
#    linkus => { type => \&sub, ..., DEFAULT => \&sub}
# + type2ref callback: &$sub($fs,$type,$node,\%node2ref_in_progress)
# + linkus callback: &$sub($fs,$qfrom,$ftr,$qto,\%node2ref);
*fs2perl = \&spit;
*as_perl = \&spit;
sub spit {
  my ($fs,%opts) = @_;

  $opts{from} = $fs->root unless (defined($opts{from}));
  return undef unless (defined($opts{from}));

  my %type2ref =
    (exists($opts{type2ref})
     ? (%SPIT_TYPE2REF, %{$opts{type2ref}})
     : (%SPIT_TYPE2REF));
  my %linkus =
    (exists($opts{linkus})
     ? (%SPIT_LINKUS, %{$opts{linkus}})
     : (%SPIT_LINKUS));

  # set up references
  my @nodes = $fs->reachable_nodes($opts{from});
  my %node2ref = ();
  my ($q,$t);
  foreach $q (@nodes) {
    $t = $fs->theta($q);
    if (defined($t) && exists($type2ref{$t})) {
      # user-defined reference-generation?
      $node2ref{$q} = &{$type2ref{$t}}($fs,$t,$q,\%node2ref);
    } else {
      # use the defaults...
      $node2ref{$q} = &{$type2ref{DEFAULT}}($fs,$t,$q,\%node2ref);
    }
  }

  # map transitions
  my ($f);
  foreach $q (@nodes) {
    $t = $fs->theta($q);
    foreach $f ($fs->features($q)) {
      if (defined($t) && exists($linkus{$t})) {
	# user-defined reference-generation?
	&{$linkus{$t}}($fs,$q,$f,$fs->delta($q,$f),\%node2ref);
      } else {
	# use the defaults...
	&{$linkus{DEFAULT}}($fs,$q,$f,$fs->delta($q,$f),\%node2ref);
      }
    }
  }
  return $node2ref{$opts{from}};
}


#sub _linkus_fsarray {
#  my ($fs,$from,$f,$to,$node2ref) = @_;
#  if (UNIVERSAL::isa($_[4]->{$_[1]},'ARRAY')) {
#    # we can now assume it's good
#    $_[4]->{$_[1]}->[$_[2]] = $_[4]->{$_[3]};
#  } else {
#    # do the (old) default thing...
#    Data::FStructure::perldata::_linkus_default(@_);
#  }
#}



###############################################################
# gobble
###############################################################

# $node = $fs->gobble($any)
#       ---> gobbles $any at root
# $node = $fs->gobble($any,%opts)
# %opts: (
#         from => $from_nodeid, : gets changed!
#         at => $at_feature,    : gets changed!
#         as => $fstype,
#         share_scalars => $GOBBLE_SHARE_SCALARS,
#         call_coderefs => $GOBBLE_CALL_CODEREFS,
#         call_methods => $GOBBLE_CALL_METHODS,
#         untie_hashes => $GOBBLE_UNTIE_HASHES
#         untie_arrays => $GOBBLE_UNTIE_ARRAYS
#         want_linked_lists => $GOBBLE_WANT_LINKED_LISTS,
#         use_approp => $GOBBLE_USE_APPROP,
#         reuse_paths = $GOBBLE_REUSE_PATHS,
#         val2node => HashRef
#        )
#       ---> gobbles $any into fs at $ftr
*perl2fs = \&gobble;
*from_perl = \&gobble;
sub gobble {
  my ($fs,$data,%opts) = @_;

  # set up options
  $opts{share_scalars} = $GOBBLE_SHARE_SCALARS unless (exists($opts{share_scalars}));
  $opts{call_coderefs} = $GOBBLE_CALL_CODEREFS unless (exists($opts{call_coderefs}));
  $opts{call_methods} = $GOBBLE_CALL_METHODS unless (exists($opts{call_methods}));
  $opts{untie_hashes} = $GOBBLE_UNTIE_HASHES unless (exists($opts{untie_hashes}));
  $opts{untie_arrays} = $GOBBLE_UNTIE_ARRAYS unless (exists($opts{untie_arrays}));
  $opts{use_approp} = $GOBBLE_USE_APPROP unless (exists($opts{use_approp}));
  $opts{reuse_paths} = $GOBBLE_REUSE_PATHS unless (exists($opts{reuse_paths}));
  $opts{want_linked_lists} = $GOBBLE_WANT_LINKED_LISTS unless
    (exists($opts{want_linked_lists}));

  $opts{use_approp} = '' unless (defined($fs->hierarchy));

  my $stack = [$opts{from},$opts{at},$data];
  my $any2node =
    defined($opts{val2node})
      ? $opts{val2node}
      : ($opts{val2node} = {});

  # ...and gobble
  my ($any,$type);
  while (($opts{from},$opts{at},$any) = splice(@$stack,0,3)) {

    # can't gobble undef
    next if (!defined($any));

    # ... no `from' specified? --> from the root, then
    $opts{from} = $fs->root unless (defined($opts{from}));

    if (ref($any)) {
      if ($opts{call_methods} && UNIVERSAL::can($any,'_be_gobbled')) {
	# oo user-gobbling hook
	$any->_be_gobbled($fs,$any2node,$stack,\%opts);
	next;
      }
      elsif ($opts{call_coderefs} && UNIVERSAL::isa($any,'CODE')) {
	# fcnl user-gobbling hook
	&$any($any,$fs,$any2node,$stack,\%opts);
	next;
      }

      # don't multiply gobble other references
      next if (exists($any2node->{$any}) && defined($any2node->{$any}));

      if (UNIVERSAL::isa($any,'HASH')) {
	if ($opts{untie_hashes} && tied(%$any)) {
	  $any = tied(%$any);
	  redo;
	} else {
	  # actually gobble the hash
	  $fs->_gobble_hash($any,$any2node,$stack,\%opts);
	}
      }
      elsif (UNIVERSAL::isa($any,'ARRAY')) {
	if ($opts{untie_arrays} && tied(@$any)) {
	  $any = tied(@$any);
	  redo;
	} else {
	  if ($opts{want_linked_lists} && $fs->can('_gobble_list')) {
	    # gobble the array as a linked list
	    $fs->_gobble_list($any,$any2node,$stack,\%opts);
	  } elsif ($fs->can('_gobble_array')) {
	    # gobble the array as an array
	    $fs->_gobble_array($any,$any2node,$stack,\%opts);
	  }  else {
	    # don't know how
	    $fs->_gobble_ref($any,$any2node,$stack,\%opts);
	  }
	}
      }
      else {
	# gobble other references
	$fs->_gobble_ref($any,$any2node,$stack,\%opts);
      }
    } else {
      # gobble simple scalars (possibly more than once)
      $fs->_gobble_scalar($any,$any2node,$stack,\%opts);
    }
  } continue {

    if (exists($any2node->{$any})) {

      if (defined($opts{from}) && defined($opts{at})) {

	# set up transition for $any (if possible)
	$fs->delta($opts{from},$opts{at},$any2node->{$any});

	# set up types -- user request?
	if ($any eq $data && exists($any2node->{$data}) && exists($opts{as})) {
	  $fs->theta($any2node->{$data},$opts{as});
	}
	elsif ($opts{use_approp} && defined($fs->theta($opts{from}))) {
	  # do the appropriate thing (if we must...)
	  $type =
	    $fs->hierarchy->njoin
	      ($fs->theta($any2node->{$any}),
	       $fs->hierarchy->get_approp_det($fs->theta($opts{from}),$opts{at}));
	  $fs->theta($any2node->{$any},$type)
	    unless (defined($type) && $type eq $TYPE_TOP); # ignore TOP
	}
      }
      else {
	# ... no root *or* feature defined? --> must *be* the root
	$fs->root($any2node->{$any});
      }
    }
  }
  return $fs;
}

# $gobbled_fs->_be_gobbled($into_fs,$val2node,\@stack,\%opts)
sub _be_gobbled {
  my ($gobbled,$gobbler,$any2node) = @_;

  if ($gobbler->is_eq($gobbled)) {
    # ... we have a winner
    $any2node->{$gobbled} = $gobbled->root if (defined($any2node));
    return;
  }
  # make a *real* node-distinct copy
  my $gobbled2 = $gobbled->clone()->alienate($gobbler);

  # update nodes
  $gobbler->{Qset}->_union($gobbled2->{Qset});

  # update type-map (safe since alien)
  $gobbler->{theta}->merge($gobbled2->{theta});

  # update node-map (safe, since alienated)
  $gobbler->{delta}->_merge($gobbled2->{delta});

  # set node-value to copied root node
  $any2node->{$gobbled} = $gobbled2->root if (defined($any2node));
}


# $fs->_gobble_hash($hashref,\%val2nodes,\@stack,\%opts)
sub _gobble_hash {
  my ($fs,$any,$any2node,$stack,$opts) = @_;

  # need new node if _gobble_hash() was called...
  $any2node->{$any} = $fs->_gobble_get_node($opts);

  # set up type assignment for blessed references ONLY
  $fs->theta($any2node->{$any}, ref($any)) if (ref($any) ne 'HASH');

  # gobble the hashref
  push(@$stack, map { ($any2node->{$any},$_,$any->{$_}) } keys(%$any));
}


# $fs->_gobble_array($aryref,\%val2nodes,\@stack,\%opts)
sub _gobble_array {
  my ($fs,$any,$any2node,$stack,$opts) = @_;

  unless ($fs->isa('Data::FStructure::Array')) {
    # oops -- autopromote
    bless($fs, 'Data::FStructure::Array');
    bless($fs->{delta}, 'Data::FStructure::arydelta');
  }

  # need new node if _gobble_array() was called...
  $any2node->{$any} = $fs->_gobble_get_node($opts);

  # set up type assignment for (blessed) references
  $fs->theta($any2node->{$any}, ref($any)) if (ref($any) ne 'ARRAY');

  # gobble the arrayref
  my ($i);
  for ($i = 0; $i <= $#$any; ++$i) {
    push(@$stack, ($any2node->{$any},$i,$any->[$i]));
  }
}


# $fs->_gobble_ref($ref,\%vals2nodes,\@stack,\%opts)
sub _gobble_ref {
  my ($fs,$any,$any2node,$stack,$opts) = @_;

  # need new node if _gobble_array() was called...
  $any2node->{$any} = $fs->_gobble_get_node($opts);

  # ... any give it a value (scalars only)
  if (UNIVERSAL::isa($any, 'SCALAR')) {
    # set up type assignment for blessed references
    $fs->theta($any2node->{$any}, ref($any));
    push(@$stack, $any2node->{$any},$GOBBLE_REFVAL_FTR,$$any);
  } else {
    # just set the type...
    $fs->theta($any2node->{$any}, $any);
  }
}


# $fs->_gobble_scalar($scalar,\%vals2nodes,\@stack,\%opts)
# ---> shares scalar values if $opts{share_scalars} is true.
sub _gobble_scalar {
  my ($fs,$any,$any2node,$opts) = @_[0,1,2,4];
  # share scalars
  return if ($opts->{share_scalars} && exists($any2node->{$any}));

  # just add node, set scalar value to its 'type'
  $any2node->{$any} = $fs->_gobble_get_node($opts);
  $fs->theta($any2node->{$any}, $any);
}

# $node = _gobble_get_node(\%opts)
sub _gobble_get_node {
  my ($fs,$opts) = @_;
  if (!$opts->{reuse_paths}) { return $fs->add_node; }

  # we're re-using paths: do we have this one?
  $opts->{from} = $fs->root unless (defined($opts->{from}));
  # still not defined? -- need root
  $fs->root($opts->{from} = $fs->add_node) unless (defined($opts->{from}));

  my ($node);
  if (defined($opts->{at})) {
    $node = $fs->delta($opts->{from},$opts->{at});
    $node = $fs->add_node unless (defined($node));
  }
  else {
    $node = defined($opts->{from}) ? $opts->{from} : $fs->add_node;
  }
  return $node;
}


1;
__END__

# -*- Mode: Perl -*-
=pod

=head1 NAME

Data::FStructure::perldata -
Feature structures for perl: convert to/from standard perl
data-structures.

=head1 SYNOPSIS

  use Data::FStructure::perldata;

  #
  # Package Variables
  #
  $GOBBLE_SHARE_SCALARS = 1;    # are scalars atoms?
  $GOBBLE_CALL_CODEREFS = 1;    # eval user code?
  $GOBBLE_CALL_METHODS = 1;     # eval '_be_gobbled()' methods?
  $GOBBLE_UNTIE_HASHES = 1;     # gobble tied hashrefs as underlying structure?
  $GOBBLE_UNTIE_ARRAYS = 1;     # gobble tied arrayrefs as underlying structure?
  $GOBBLE_REFVAL_FTR = 'VALUE'; # what do we call the dereference feature?

  #
  # Construction & Initialization
  #
  $fs = Data::FStructure::Base->new(\%args);

  #
  # High-Level Conversion Methods
  #
  $fs->gobble($perldata);       # convert perl data-structure to fs
  $fs->spit();                  # convert fs to perl data-structure

  #
  # API
  #
  $fs->_be_gobbled($intofs,            # used for substructure assignment
		   \%val2node,
		   \@stack,
		   \%opts);

  $ref = _type2ref_default($fs,$type); # used for default spit() operations
  $ref = _linkus_default($fs,
			 $qfrom,
			 $ftr,
			 $qto,
			 $node2ref);

=head1 REQUIREMENTS

Carp,
Exporter,
Data::FStructure::Base.

=cut

#--------------------------------------------------------------
# Description
#--------------------------------------------------------------
=pod

=head1 DESCRIPTION

C<Data::FStructure::perldata> provides methods in the package
C<Data::FStructure::Base> for reading standard perl data-structures
into FStructure objects, and for reading FStructure objects
out as standard perl data structures.

The high-level method C<gobble()> is responsible for
incorporating perl data-structures into FStructure objects.
Its counterpart, the C<spit()> method, is responsible for
constructing perl data-structures out of FStructure
objects.

=cut

#--------------------------------------------------------------
# Variables
#--------------------------------------------------------------
=pod

=head1 PACKAGE VARIABLES

C<Data::FStructure::perldata> defines the following variables,
and imports them into C<Data::FStructure::Base>.

=over 4

=item * C<$GOBBLE_SHARE_SCALARS>

Whether simple perl scalars should be added at most once
by default during C<gobble()> operations.
Default is C<1> (yes).

=item * C<$GOBBLE_CALL_CODEREFS>

Whether C<CODE> references should be called by default
during C<gobble()> operations.  Default is C<1> (yes).

=item * C<$GOBBLE_CALL_METHODS>

Whether the method C<_be_gobbled()>
should be called by default during C<gobble()>
operations for classes (or blessed references) which
define it.  Default is C<1> (yes).

=item * C<$GOBBLE_UNTIE_HASHES>

Whether C<gobble()> operations should work on the
underlying structure of tied hash-references, rather
than on the apparent structure of the tied hash by default.
Default is C<1> (yes).

=item * C<$GOBBLE_UNTIE_ARRAYS>

Whether C<gobble()> operations should work on the
underlying structure of tied hash-references, rather
than on the apparent structure of the tied hash by default.
Default is C<1> (yes).

=item * C<$GOBBLE_WANT_LINKED_LISTS>

Whether C<gobble()> and C<spit()> operations should build linked
lists out of C<ARRAY> references, rather than setting
numerical feature-values.  Default is C<''> (no).


=item * C<$GOBBLE_REFVAL_FTR>

The name of a feature to which the values of other
references will be assigned by default during
C<gobble()> operations.  Default is C<VALUE>.

=item * C<%SPIT_TYPE2REF>

A hash of code-references which will be called to generate
perl-references based on the type assigned to an FStructure
node during C<spit()> operations.

=item * C<%SPIT_LINKUS>

A hash of code-references which will be called to convert
assign feature-transitions in perl-references during
C<spit()> operations.

=back

=cut

#--------------------------------------------------------------
# Methods
#--------------------------------------------------------------
=pod

=head1 METHODS

=cut


#--------------------------------------------------------------
# Gobble Methods
#--------------------------------------------------------------
=pod

=head2 From perl to FStructures: gobble()

Calling conventions: C<< $fs-E<gt>gobble($data,OPTION=E<gt>VALUE,...) >>

Aliases: C<perl2fs()>, C<from_perl()>.

Reads the standard perl data-structure C<$data> into the FStructure
C<$fs>.

Recognizes the following options:

=over 4

=item * C<< from =E<gt> $nodeid >>

Read C<$data> into the FStructure from $nodeid, which defaults
to the root of the FStructure, or a new node which will be
set to the root of the FStructure if it is currently unrooted.

=item * C<< at =E<gt> $at_feature >>

Read C<$data> into the FStructure as the value of the
C<$at_feature> from the node indicated by the
C<from> argument.  If C<at> is not specified, the
FStructure will be (re-)rooted at a new node representing
the data read in.

=item * C<< share_scalars =E<gt> $bool >>

Whether or not scalar values should be shared.  Default
is C<$GOBBLE_SHARE_SCALARS>.

=item * C<< call_coderefs =E<gt> $bool >>

Whether or not C<CODE> references should be evaluated.
Default is C<$GOBBLE_CALL_CODEREFS>.

=item * C<< call_methods =E<gt> $bool >>

Whether or not the instance method C<_be_gobbled()> should be
called if it is available.  Default is
C<$GOBBLE_CALL_METHODS>.

=item * C<< untie_hashes =E<gt> $bool >>

Whether or not tied C<HASH> references should be
untied before they are gobbled. Default is
C<$GOBBLE_UNTIE_HASHES>.

=item * C<< untie_arrays =E<gt> $bool >>

Whether or not tied C<ARRAY> references should be
untied before they are gobbled. Default is
C<$GOBBLE_UNTIE_ARRAYS>.

=item * C<< want_linked_lists =E<gt> $bool >>

Whether or not C<ARRAY> references should be
gobbled as linked lists if possible.  Default
is C<$GOBBLE_WANT_LINKED_LISTS>.

=item * C<< val2node =E<gt> $val2node >>

C<$val2node> is a hash-reference in which the mappings from
gobbled values to node-names will be stored.  Default
is a new empty hashref.

=back

C<$data> is gobbled according to the following rules:

=over 4

=item 1.

If C<call_methods> is true, and if
C<$data> is a reference to an object blessed
into a package defining or inheriting a method
named C<_be_gobbled>, then that method will be called
as:

  $data->_be_gobbled($fs,$val2node,\@stack,\%opts);

C<$fs> is the gobbling FStructure,
C<$val2node> is a hashref as described above,
C<\@stack> is a stack of triples (not nested)
of the form C<($node,$at,$data)> waiting to be processed,
and C<\%opts> is a reference to a hash containing the
options passed to the C<gobble()> method.

The method is expected to set the value
C<< $val2node-E<gt>{$data} >> to the name of the node
which it should be assigned and add this node if required,
as well as set the type of this node.
Transitions are created by the C<gobble()> method itself,
but the values of C<< $opts-E<gt>{from} >> and
C<< $opts-E<gt>{at} >> will be set to the relevant
values during the call.

This interface is used for substructure assignment by
the C<pvalue()> method of C<Data::FStructure::Base>, if it is
available.

=item 2.

If C<$data> is a C<CODE> reference and the C<call_coderefs>
option is set, then C<$data> is called as:

  &$data($data,$fs,$val2node,\@stack,\%opts);

Arguments and caveats as for the C<_be_gobbled()> method,
described above.

=item 3.

If C<$data> is any other reference for which a value exists
in C<\%val2node>, only a transition to its indicated node
is added.

=item 4.

Otherwise, if C<$data> is a reference to a tied
C<HASH>, and the C<untie_hashes> option is set, then
the tied object itself is gobbled.

If C<untie_hashes> is not set, or if C<$data> is not
a non-tied C<HASH> reference, then a new node is added
for it, and its keys are mapped to features whose
values are pushed onto the stack to be gobbled.

The C<type()> of $data is set to C<ref($data)>.

=item 5.

If C<$data> is a reference to a tied
C<ARRAY>, and the C<untie_arrays> option is set, then
the tied object itself is gobbled.

If C<untie_arrays> is not set, or if C<$data> is not
a non-tied C<ARRAY> reference, and C<want_linked_lists>
is set to a true value, then the method

  $fs->_gobble_list($any,$val2node,\@stack,\%opts)

is called if available.  Such a method is defined
by the C<Data::FStructure::List> module.

Otherwise, a new node is created for the array,
and it is assigned numeric features representing
its indices, whose values are gobbled as the
values of those index-features.  Such numeric feature-assignments
are incorporated into FStructures inheriting from
C<Data::FStructure::Array> as internal perl
array operations.

The C<type()> of $data is set to C<ref($data)>.

=item 6.

If C<$data> is any other reference, a new node is
created for it and its value is gobbled as the
value of the feature C<$GOBBLE_REFVAL_FTR>.

The C<type()> of $data is set to C<ref($data)>.

=item 7.

If C<$data> is not a reference, it must be a simple
scalar -- it will be considered node-identical to
string-identical (C<eq>) scalars of C<share_scalars>
is set.

If there is no already-gobbled datum which is string-identical
to C<$data>, or if C<share_scalars> is false, then
a new node is created for $data, and $data is assigned
to the type of that node.

=back

=cut

#--------------------------------------------------------------
# Spit Methods
#--------------------------------------------------------------
=pod

=head2 From FStructures to perl: spit()

Calling conventions: C<< $ref = $fs-E<gt>spit(OPTION=E<gt>VALUE,...) >>

Aliases: C<fs2perl()>, C<as_perl()>.

Constructs a perl data structure from the FStructure C<$fs>.

Recognizes the following options:

=over 4

=item * C<< from =E<gt> $node >>

The node of the FStructure from which to construct a
perl reference. Default is the root node.

=item * C<< type2ref =E<gt> $hashref >>

A reference to a hash of code-references keyed by type-names.
A coderef in this hash will be called once for each node in the
FStructure whose type is string-identical (C<eq>) with that
type.  The special key C<DEFAULT> will be called for nodes
whose types do not literally match any other key.  The
subroutines are called as:

  &$sub($fs,$type,$node,\%node2ref);

C<\%node2ref> is a hashref mapping already-traversed
node-names to perl references.
Subroutines are expected
to return a reference to which nodes mapped to C<$type> should
be mapped.

=item * C<< linkus =E<gt> $hashref >>

A hashref as for C<type2ref>, but coderefs are called to
generate feature-transitions.  Subroutines are called as:

  &$sub($fs,$qfrom,$ftr,$qto,\%node2ref);

=back

The default settings map nodes of type C<'ARRAY'> to perl
array-references, and nodes of any other type to hash-references.
If a type is defined for a node, the reference generated is
by default blessed into that type.

=cut


#--------------------------------------------------------------
# API
#--------------------------------------------------------------
=pod

=head2 API

C<Data::FStructure::perldata> defines the method C<_be_gobbled()> as
described above for substructure assignment in FStructures.
See the section on C<gobble()> for details.

The subroutines C<_type2ref_default> and
C<_linkus_default> handle the default conversions from
FStructures to arrays.
See the section on C<spit()> for details.

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head1 BUGS

This whole module should not be necessary -- the FStructure
methods ought to be able to work directly with any perl
data structure.

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2001, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).
Data::FStructure(3pm).
Data::FStructure::Base(3pm).
Data::FStructure::Array(3pm).
Data::FStructure::List(3pm).
