# -*- Mode: Perl -*-
# t/06_gobl.t : test gobble() and spit()

$TEST_DIR = './t';
#use lib qw(../blib/lib); $TEST_DIR = '.'; # for debugging

# load what we'll need
use Data::FStructure;
use Data::FStructure::pretty;
use Data::FStructure::perldata;
do "$TEST_DIR/common.plt";

plan(test => 17);

# prep
$fs = Data::FStructure->new;

#==================================
# gobble
#==================================

#------------------
# hash
#------------------
# 1
$hash = {a=>{b=>1,c=>1}};
$fs->gobble($hash,share_scalars=>1);
isok('gobble [hash]',$fs->as_string,'[ a: [ b: [ =1 (1) ] c: [ =1 ] ] ]');

#------------------
# array
#------------------
# 2
$ary = [qw(a b c)];
$fs->gobble($ary, want_linked_lists=>1);
isok('gobble [array:want_linked_lists=>1]',
     $fs->as_string,
     '[ first: [ (a) ] next: [ first: [ (b) ] next: [ first: [ (c) ] ] ] ]');

#------------------
# list
#------------------
# 3
$fs->clear();
$fs->gobble($ary, want_linked_lists=>0);
isok('gobble [array:want_linked_lists=>0]',
     $fs->as_string,
     '[ 0: [ (a) ] 1: [ (b) ] 2: [ (c) ] ]');


#==================================
# spit
#==================================

#------------------
# hash
#------------------
# 4
$fs->from_text('[ moo: [ (cow) ] cow: [ (bovine) ] ]');
$hr = $fs->spit;
ulistok('spit [keys]', [keys(%$hr)], [qw(moo cow)]);
# 5-6
ulistok('spit [keys:moo]', [keys(%{$hr->{moo}})], []);
ulistok('spit [keys:cow]', [keys(%{$hr->{cow}})], []);
# 7
ulistok('spit [ref]', ref($hr), 'HASH');
# 8-9
ulistok('spit [ref:moo]', ref($hr->{moo}), 'cow');
ulistok('spit [ref:cow]', ref($hr->{cow}), 'bovine');


#------------------
# array
#------------------
# 10
$fs->from_text('[ 0: [ (moo) ] 1: [ (cow) ]]');
$ar = $fs->spit;
isok('spit [length]', $#$ar, 1);
# 11
ulistok('spit [ref]', ref($ar), 'ARRAY');
# 12-13
ulistok('spit [ref:0]', ref($ar->[0]), 'moo');
ulistok('spit [ref:1]', ref($ar->[1]), 'cow');


#------------------
# list
#------------------
# 14
$fs->from_text('[ (mylist) first: [ (moo) ] next: [ (mylist) first: [ (cow) ] ] ]');
$ar = $fs->spit(type2ref => { mylist => \&Data::FStructure::Base::_type2ref_list },
		linkus => { mylist => \&Data::FStructure::Base::_linkus_list });
isok('spit [length]', $#$ar, 1);
# 15
ulistok('spit [ref]', ref($ar), 'ARRAY');
# 16-17
ulistok('spit [ref:0]', ref($ar->[0]), 'moo');
ulistok('spit [ref:1]', ref($ar->[1]), 'cow');


print "\n";
# end of t/06_gobl.t

