# -*- Mode: Perl -*-
# File: t/common.plt
# Description: re-usable test subs for Data::FStructure
use Test;
$| = 1;

sub safestr {
  return defined($_[0]) ? "'$_[0]'" : 'undef';
}


# isok($label,@_) -- prints helpful label
sub isok {
  my $label = shift;
  print "$label:\n";
  ok(@_);
}

# ulistok($label,\@got,\@expect)
# --> ok() for unsorted lists (no ',' allowed in elements!)
sub ulistok {
  my ($label,$l1,$l2) = @_;
  isok($label,join(',',sort(@$l1)),join(',',sort(@$l2)));
}

# slistok($label,\@got,\@expect)
# --> ok() for sorted lists (no ',' allowed in elements!)
sub slistok {
  my ($label,$l1,$l2) = @_;
  isok($label,join(',',@$l1),join(',',@$l2));
}


# hashok($label,\%got,\%expect)
# --> ok() for hashrefs (no ',' or '=>' allowed in elements!)
sub hashok {
  my ($label,$h1,$h2) = @_;
  isok($label,
       join(',',
	    (map { "$_=>$h1->{$_}" } keys(%$h1)),
	    (map { "$_=>$h2->{$_}" } keys(%$h2))));
}


# $hi = truth_hi()  -- test hierarchy
sub truth_hi {
  eval "use Math::PartialOrder::Set";
  my $h = Math::PartialOrder::Set->new({root => 'whatever'});
  $h->add(qw(yup whatever));
  $h->add(qw(nope whatever));
  $h->add(qw(maybe yup nope));
  $h->add(qw(maybenot yup nope));
  $h->add(qw(maybenot yup nope));
  $h->add(qw(whoknows maybe maybenot));
  $h->add(qw(whocares whatever));
  return $h;
}

# $hi = attitudes_hi()  -- another test hierarchy
sub attitudes_hi {
  eval "use Math::PartialOrder::Set;";
  eval "use Data::FStructure::Approp;";
  my $h = Math::PartialOrder::Set->new({root => 'BOTTOM'});

  $h->add(qw(value));
  $h->add(qw(true value));
  $h->add(qw(false value));
  $h->add(qw(maybe true false));
  $h->add(qw(maybenot true false));
  $h->add(qw(whoknows maybe maybenot));

  $h->add(qw(attitude));
  $h->add(qw(assertion attitude));
  $h->add(qw(denial attitude));
  $h->add(qw(question attitude));
  $h->add(qw(skepsis attitude));
  $h->add(qw(curiosity skepsis question));

  $h->set_appr_bytype(qw(assertion ARG true));
  $h->set_appr_bytype(qw(denial ARG false));
  $h->set_appr_bytype(qw(question ARG maybe));
  $h->set_appr_bytype(qw(skepsis ARG maybenot));
  #$h->set_appr_bytype(qw(curiosity ARG whoknows)); # - this should be computed

  $h->add(qw(contradiction assertion denial));
  #$h->set_appr_bytype(qw(contradiction ARG maybe|maybenot)) # - nondet: goofy (no syntax,either)

  return $h;
}


print "common.plt loaded.\n";

1;

